/***************************************************************************/
/**
	@file		cmd_interface.c
	@brief		Simple implementation of an command-interface
	@brief		Target/Compiler: AVR_MEGA / GCC(WinAVR)
	@author		Gerhard Bruenner
	@version	V2.03

	@date		2009.12.29  - GB
				created new function split_command_line() - now using own version of strtok.
				This reduced the operating-time of this function to the half of the previous version.  
				Although changed the function-parameter interface of the user-function to
				uint8_t		cmd_func (int8_t argc, char *argval[])
	@date		2009.11.15  - added function cmd_interface_add_cmd and Backspace-Function - GB
	@date		2009.10.31  - changed the way how commandline is interpret - GB
	@date		2009.10.21  - small updates   - GB
	@date		2009.02.01  - Created         - GB

	@bug		none that I am aware so far

\***************************************************************************/

#define   CMD_INTERFACE_C

/* Include-Dateien
   ************************************************/
	#include "cmd_interface.h"


/* globale int Variablen
   ************************************************/
			char   	c_ptr_cmd_line[_CMD_INTERFACE_MAX_LENGTH+1];
	static 	int8_t	cmd_line_act_length=0;
	static 	char 	*cmd_line_split_parts[_CMD_INTERFACE_MAX_NO_OPERANDS];
	cmd_interface_command_table_t cmd_interface_table[_CMD_INTERFACE_NO_OF_COMMANDS];

/* externe Variablen
   ************************************************/


/* internal Funktions-Prototypen
****************************************************************************/
	int8_t execute_command();
	int8_t split_command_line();

/**
 	@brief		initialisation routine for command interface variables
 	@param		none
 	@return		void
 */
void cmd_interface_init(){
	register uint8_t ui8_cnt=0;
	for(ui8_cnt=0; _CMD_INTERFACE_NO_OF_COMMANDS>ui8_cnt; ui8_cnt++) {
		cmd_interface_table[ui8_cnt].cmd=_STRING_EOS;
		cmd_interface_table[ui8_cnt].cmd_func_ptr=NULL;
	}
}/* void cmd_interface_init()*/

/**
 	@brief		adding a command to internal command - table

 	@param		char *cmd_string 		- String representing the command
 	@param		uint8_t (*cmd_func_ptr) - pointer to function which shall be executed if the command has been found

 	@return		 0 - all fine
 				-1 - command-table is full
 	@details	This function shall add a given string and function-pointer to the internal command-table.
 				The string in the command-table is used by the internal functions to determine whether
 				a command is known or not. The function where the function-pointer is pointing to is called in
 				the case the entered command matches to the given string.
 				The function MUST have the following interface-declaration....
 				uint8_t		cmd_func (int8_t argc, char *argval[])
 */
uint8_t cmd_interface_add_cmd(char (*cmd_string), uint8_t (*cmd_func_ptr)(int8_t argc, char *argval[]) ){
	static uint8_t ui_static_cmd_counter=0;
	uint8_t ret_val = -1; /* ERROR: -1 >> Command-Table is full */

	if(_CMD_INTERFACE_NO_OF_COMMANDS>=ui_static_cmd_counter){ 					/* there are still commands left to be set */
		trim(&cmd_string[0],_ASCII_SPACE); 										/* remove SPACES at the beginn and at the End */
		cmd_interface_table[ui_static_cmd_counter].cmd=cmd_string;				/* add command-string to table */
		cmd_interface_table[ui_static_cmd_counter].cmd_func_ptr=cmd_func_ptr;	/* add function-pointer to table */
		ui_static_cmd_counter++; /* inkrement command-counter */
		ret_val =0; /* clear preset errror */
	}

	return ret_val;
}/* cmd_interface_add_cmd */

/**
 	@brief		routine called to execute command interface
 	@param		*string - to attach at the actual command_line buffer
	@return		The content of the return-value have been split into two parts
	@return		The three lowest bits are used as counter for executed commands
	@return		The remaining "upper" bits represent bit-coded errors,
	@return		due to the possibilitiy of multiple errors per call.
 	@return		  <0 		- error occured
 	@return		#1001 0XXX	- #84	- not used
 	@return		#1001 0XXX	- #90	- too many operands in the command-line
 	@return		#1010 0XXX	- #A0	- no valid command found
 	@return		#1100 0XXX	- #B0	- too many signs in comand-line
 	@return		  0			- no end of command-line detects
 	@return		 0> 		- number of commands which have been executed
 	@details	This routine has to be called to execute the command-interface
 				It attaches the forwarded string to the internal command-line buffer
 				and its sub-functions interprets the line before it executes the corresponding
 				command-function given by the user
 */
int8_t execute_cmd_interface(char *string){
	/* local variables */
	register uint8_t	ui8_cnt		=0;
	register char		act_char	=1;
	register int8_t		cmd_ret_val	=0;
	register uint8_t	cmd_count	=0;
	int8_t 				ret_val		=0;
	/* start reading string */
	act_char=string[ui8_cnt++];	/* Read next sign */

	/* adding string to cmd_line while looking for an EOL */
	while	( 		(cmd_line_act_length<_CMD_INTERFACE_MAX_LENGTH)
				&&	(act_char!=_STRING_EOS)									/* EOS ? */
	){
		if(_CMD_INTERFACE_EOL==act_char){		/* End of commandline ? */
			c_ptr_cmd_line[cmd_line_act_length]=_STRING_EOS; 	/* Set EOS in string of commandline */
			cmd_ret_val=execute_command();
			switch (cmd_ret_val) {
				case  0:						/* set error: no valid command */
						ret_val |= _CMD_INTERFACE_ERR_NO_CMD;
						break;
				case -1:						/* set error: operands have to many characters */
						ret_val |= _CMD_INTERFACE_ERR_OP_2_MANY;
						break;
				default:						/* count up counter for executed commands */
						if (0<cmd_ret_val)	cmd_count++;
						break;
			} /* switch (cmd_ret_val)*/
		} else if (_ASCII_BS==act_char){
			cmd_line_act_length--;
		} else {
			c_ptr_cmd_line[cmd_line_act_length++]=act_char;
		}
		act_char=string[ui8_cnt++];		/* Read next sign */
	} /* while */

	/* to many signs in commandline ? */
	if (cmd_line_act_length>=_CMD_INTERFACE_MAX_LENGTH){
		cmd_line_act_length=0;
		/* set error: to many signs in commandline */
		ret_val |= _CMD_INTERFACE_ERR_CMD_LINE;
	}

	/* Set EOS in string of commandline */
	c_ptr_cmd_line[cmd_line_act_length]=_STRING_EOS;

	/* combine ret_val-information */
	ret_val |= cmd_count& ~_CMD_INTERFACE_ERR_MASK;

	return ret_val;
} /*execute_cmd_interface()*/


/**
 	@brief		internal function which executes the commands
 	@param		none
 	@return		<0 	- error
	@return		-1	- too many operands in the command-line (ret_val of split_command_line)
	@return		 0 	- no valid command
	@return		 1	- a command hast been successfully executed.
 	@details    this functions checks whether the given command exists in the command-table
 				if it exists it executes the function referenced in the function-pointer
 				which belongs the specific command.
 */
int8_t execute_command(){
	uint8_t ui8_cnt=0;
	int8_t  ret_val=0;
	int8_t	i8_ret_split;
	/* Analyze command_line and transfer segmented data into data-array */
		i8_ret_split=split_command_line();
		if (0>i8_ret_split){
		 ret_val=i8_ret_split;
		 goto func_end;
		}
	/* check whether the command is valid and execute the corresponding function */
		for(ui8_cnt=0; _CMD_INTERFACE_NO_OF_COMMANDS>ui8_cnt; ui8_cnt++){
			if ( cmd_interface_table[ui8_cnt].cmd_func_ptr != NULL )  {
				if ( strcmp( cmd_interface_table[ui8_cnt].cmd,cmd_line_split_parts[0] )==0 ) {
					cmd_interface_table[ui8_cnt].cmd_func_ptr(--i8_ret_split, &cmd_line_split_parts[1]);
					ui8_cnt = _CMD_INTERFACE_NO_OF_COMMANDS; /* skip rest - only one command possible per line */
					ret_val = 1; /* one cmd executed */
				}
			} /* if ( cmd_interface_table[ui8_cnt].cmd_func_ptr != NULL ) */
		}
func_end:
	/* clean command line */
		cmd_line_act_length			=0;
	return ret_val;
}/*inline int8_t execute_command()*/



/**
 	@brief		internal function which splits the act. command line into separated components, which are
 				separated by split signs.
 	@param		none
 	@return		<0 	- 	error
 	@return		-1	-	too many operands in the command-line
 	@return		>0 	- 	number of strings/operands which have been found, means separated by split-signs.
 */
int8_t split_command_line(){
	int8_t	op_cnt			=0;		/* operand counter which shall not exceed cmd_interface_max_no_operands*/
	char	*c_ptr_found	=0;

	/* whipe previous results */
	for (op_cnt=0; op_cnt<_CMD_INTERFACE_MAX_NO_OPERANDS; op_cnt++) cmd_line_split_parts[op_cnt]=NULL; 

	/* check for command */
	trim(c_ptr_cmd_line,_ASCII_SPACE);
	c_ptr_found = strtrok_single_char(c_ptr_cmd_line, _CMD_INTERFACE_COMMAND_SPLIT_SIGN);

	/* check for operands */
	op_cnt=0; 
	while( (NULL!=c_ptr_found)&&(_CMD_INTERFACE_MAX_NO_OPERANDS>op_cnt)){
		trim(c_ptr_found,_ASCII_SPACE);
		cmd_line_split_parts[op_cnt++]=c_ptr_found;
		c_ptr_found=strtrok_single_char(NULL, _CMD_INTERFACE_OPERAND_SPLIT_SIGN);
	}

	/* check whether further operands would be available */
	if (NULL!=c_ptr_found)	op_cnt = -1;		/* error - too many operands */

	return op_cnt;
}/* int8_t execute_command()*/




#undef  CMD_INTERFACE_C



#ifdef NEVER

/** Messung zur V2.03
    Version von split_command_line() mit eigener version von strtok,
	Test-Takt 16 Mhz
	//	sprintf(str_rcv_usart,"test op1, op2, op4%c",toascii(13));
	//	i8_tmp=execute_cmd_interface(&str_rcv_usart[0]);

	strtok version	
		Start				End
		count	time[us]	count	time[us]
		8163	0			9126	63,9

	alte version
		Start				End
		count	time[us]	count	time[us]	
		11580	0			14155	160,94	
*/


//	sprintf(str_rcv_usart,"test operand1, operand2,operand3, operand4%c",toascii(13));
//	sprintf(str_rcv_usart,"test 123456789A12345, operand2,operand3, operand4%c",toascii(13));
//	sprintf(str_rcv_usart,"test op1, op2, op3, op4, op5%c",toascii(13));
//	sprintf(str_rcv_usart,"test op1, op2, op3%c",toascii(13));
//	sprintf(str_rcv_usart,"test ,,,,%c",toascii(13));
//				i8_tmp=execute_cmd_interface(&str_rcv_usart[0]);


#endif


