/***************************************************************************\

  File........: ringbuffer.c
  Author(s)...: Gerhard Brnner
  Target(s)...: AVR_MEGA
  Compiler....: WIN_AVR
  Description.: Routines handling an ringbuffer with varialbe access-length

  Revisions...: V2.02
  2009.01.18  - first shot
  2009.02.01  - improved code - hope this one is faster
															V1.00 		V1.01
				function add  without comp. optimisation	17,75s		28,00s
  							  with    comp. optimisation 	17,50s		10,25s  >> much faster

				function read without comp.optimisation		20,75s		30,00s
  							  with    comp.optimisation 	20,75s		14,5 s	 >> much faster
  2009.11.20  - removed toggle bit in struct _RINGBUFFER_T
  				changed name of ringbuffer_t to _RINGBUFFER_T
  2009.11.29  - add function ringbuffer_flush
  2009.12.27  - removed array-access to buffer-elements and added global defines for
  				buffer-datatype and access_width
  				removed binary-mask for swap at boundry of data-array
  2009.12.27  - added - binary mask for boundry changes again (	buf->WritePos&=~_RINGBUFFER_MAX_BUFFER_LEN;	)
  				because of performance improvements

\***************************************************************************/

#define   _RINGBUFFER_C

/* Include-Dateien
   ************************************************/
	#include "ringbuffer.h"

 /* Internal Defines
 ****************************************************************************/

 /* Function Prototypes
 ****************************************************************************/

 /* globale int Variablen
 ****************************************************************************/

 /* Functions
 ****************************************************************************/


/**
 	@brief		initializes the buffer structure and allocates the required memory
 	@param		buffer_length_in_byte - number of byte required for the buffer.
 	@return		void

 	@details    ....
 */
void ringbuffer_t_init(_RINGBUFFER_T *buf ){
	_RINGBUFFER_ACCESS_WIDTH cnt=_RINGBUFFER_MAX_BUFFER_LEN;
	buf->WritePos = buf->ReadPos = buf->Overflow  =0;
	while(0<cnt) *(buf->Data+(--cnt))=0;
} /*int8_t ringbuffer_t_init( uint16_t buffer_length_in_byte);*/


/**
 	@brief		writes data into the buffer structure
 	@param		_RINGBUFFER_T 	*buf	-	pointer on the  buffer-structure
 	@param		uint8_t 		data	-	byte which shall be written into buffer
 	@return		0	-	everything is fine
 	@return		-1	-	buffer overflow

 	@details    ....
 */
int8_t ringbuffer_add(_RINGBUFFER_T *buf, _RINGBUFFER_DATATYPE_CONTENT data) {
	*(buf->Data+buf->WritePos++)=data;
	buf->WritePos&=~_RINGBUFFER_MAX_BUFFER_LEN;
	if (buf->WritePos==buf->ReadPos){
		buf->Overflow++;
		return -1;
	}
	return	0;
} /* int8_t ringbuffer_add(_RINGBUFFER_T *buf, uint8_t data) */


/**
 	@brief		read data from the buffer structure
 	@param		_RINGBUFFER_T 	*buf	-	pointer to buffer-structure
 	@param		uint8_t 		*data	-	pointer on the memory where the read byte should be placed
 	@return		=0	-	nothing to read0	-	everything is fine
 	@return		>0	-	amount of bytes read-1	-	buffer overflow
 	@details    ....
 */

int8_t ringbuffer_read(_RINGBUFFER_T *buf, _RINGBUFFER_DATATYPE_CONTENT *data){
	if(buf->ReadPos!=buf->WritePos){
		*data=*(buf->Data+buf->ReadPos++);
		buf->ReadPos&=~_RINGBUFFER_MAX_BUFFER_LEN;
		return 1;
	}
	return 0;
} /* int8_t ringbuffer_read */


/**
 	@brief		read byte all bytes from the buffer structure
 	@param		_RINGBUFFER_T 	*buf	-	pointer to buffer-structure
 	@param		uint8_t 		*data	-	pointer on the memory array where the read bytes should be placed
 	@return		=0	-	nothing to read0	-	everything is fine
 	@return		>0	-	amount of bytes read-1	-	buffer overflow
 	@details    ....
 */

_RINGBUFFER_ACCESS_WIDTH ringbuffer_flush(_RINGBUFFER_T *buf, _RINGBUFFER_DATATYPE_CONTENT *data){
	_RINGBUFFER_ACCESS_WIDTH ret_val=0;
	while ( 0<ringbuffer_read(buf,data)){
		data++;
		ret_val++;
	}
	return ret_val;
} /* _RINGBUFFER_ACCESS_WIDTH ringbuffer_flush */


/* Testcode
char act_char=0;
ringbuffer_t_init(&glob_usart_receive_buffer);
ringbuffer_add(&glob_usart_receive_buffer,'1');
ringbuffer_read(&glob_usart_receive_buffer,&act_char);
ringbuffer_read(&glob_usart_receive_buffer,&act_char);
ringbuffer_add(&glob_usart_receive_buffer,'2');
ringbuffer_add(&glob_usart_receive_buffer,'3');
ringbuffer_add(&glob_usart_receive_buffer,'4');
ringbuffer_add(&glob_usart_receive_buffer,'5');
ringbuffer_add(&glob_usart_receive_buffer,'6');
ringbuffer_add(&glob_usart_receive_buffer,'7');
ringbuffer_add(&glob_usart_receive_buffer,'8');
ringbuffer_add(&glob_usart_receive_buffer,'x');
ringbuffer_add(&glob_usart_receive_buffer,'1');
ringbuffer_read(&glob_usart_receive_buffer,&act_char);
ringbuffer_read(&glob_usart_receive_buffer,&act_char);
ringbuffer_flush(&glob_usart_receive_buffer, (uint8_t *)&str_rcv_usart[0]);


Performance-Testcode
char act_char=0;

ringbuffer_t_init(&glob_usart_receive_buffer);
ringbuffer_add(&glob_usart_receive_buffer,'1');
ringbuffer_read(&glob_usart_receive_buffer,&act_char);
ringbuffer_add(&glob_usart_receive_buffer,'2');
ringbuffer_add(&glob_usart_receive_buffer,'3');
ringbuffer_flush(&glob_usart_receive_buffer, (uint8_t *)&str_rcv_usart[0]);

*/
